@@ -24,21 +24,21 @@ from Gnumed.business import gmDocuments
 
 _log = logging.getLogger('gm.bill')
 
-INVOICE_DOCUMENT_TYPE = u'invoice'
+INVOICE_DOCUMENT_TYPE = 'invoice'
 # default: old style
-DEFAULT_INVOICE_ID_TEMPLATE = u'GM%(pk_pat)s / %(date)s / %(time)s'
+DEFAULT_INVOICE_ID_TEMPLATE = 'GM%(pk_pat)s / %(date)s / %(time)s'
 
 #============================================================
 # billables
 #------------------------------------------------------------
-_SQL_get_billable_fields = u"SELECT * FROM ref.v_billables WHERE %s"
+_SQL_get_billable_fields = "SELECT * FROM ref.v_billables WHERE %s"
 
 class cBillable(gmBusinessDBObject.cBusinessDBObject):
 	"""Items which can be billed to patients."""
 
-	_cmd_fetch_payload = _SQL_get_billable_fields % u"pk_billable = %s"
+	_cmd_fetch_payload = _SQL_get_billable_fields % "pk_billable = %s"
 	_cmds_store_payload = [
-		u"""UPDATE ref.billable SET
+		"""UPDATE ref.billable SET
 				fk_data_source = %(pk_data_source)s,
 				code = %(billable_code)s,
 				term = %(billable_description)s,
@@ -68,7 +68,7 @@ class cBillable(gmBusinessDBObject.cBusinessDBObject):
 	]
 	#--------------------------------------------------------
 	def format(self):
-		txt = u'%s                                    [#%s]\n\n' % (
+		txt = '%s                                    [#%s]\n\n' % (
 			gmTools.bool2subst (
 				self._payload[self._idx['active']],
 				_('Active billable item'),
@@ -76,7 +76,7 @@ class cBillable(gmBusinessDBObject.cBusinessDBObject):
 			),
 			self._payload[self._idx['pk_billable']]
 		)
-		txt += u' %s: %s\n' % (
+		txt += ' %s: %s\n' % (
 			self._payload[self._idx['billable_code']],
 			self._payload[self._idx['billable_description']]
 		)
@@ -86,38 +86,40 @@ class cBillable(gmBusinessDBObject.cBusinessDBObject):
 			'perc_vat': self._payload[self._idx['vat_multiplier']] * 100,
 			'val_w_vat': self._payload[self._idx['amount_with_vat']]
 		}
-		txt += u' %s %s%s (%s)' % (
+		txt += ' %s %s%s (%s)' % (
 			self._payload[self._idx['catalog_short']],
 			self._payload[self._idx['catalog_version']],
-			gmTools.coalesce(self._payload[self._idx['catalog_language']], u'', ' - %s'),
+			gmTools.coalesce(self._payload[self._idx['catalog_language']], '', ' - %s'),
 			self._payload[self._idx['catalog_long']]
 		)
-		txt += gmTools.coalesce(self._payload[self._idx['comment']], u'', u'\n %s')
+		txt += gmTools.coalesce(self._payload[self._idx['comment']], '', '\n %s')
 
 		return txt
 	#--------------------------------------------------------
 	def _get_is_in_use(self):
-		cmd = u'SELECT EXISTS(SELECT 1 FROM bill.bill_item WHERE fk_billable = %(pk)s LIMIT 1)'
+		cmd = 'SELECT EXISTS(SELECT 1 FROM bill.bill_item WHERE fk_billable = %(pk)s LIMIT 1)'
 		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': {'pk': self._payload[self._idx['pk_billable']]}}])
 		return rows[0][0]
 
 	is_in_use = property(_get_is_in_use, lambda x:x)
 
 #------------------------------------------------------------
-def get_billables(active_only=True, order_by=None):
+def get_billables(active_only=True, order_by=None, return_pks=False):
 
 	if order_by is None:
-		order_by = u' ORDER BY catalog_long, catalog_version, billable_code'
+		order_by = ' ORDER BY catalog_long, catalog_version, billable_code'
 	else:
-		order_by = u' ORDER BY %s' % order_by
+		order_by = ' ORDER BY %s' % order_by
 
 	if active_only:
-		where = u'active IS true'
+		where = 'active IS true'
 	else:
-		where = u'true'
+		where = 'true'
 
 	cmd = (_SQL_get_billable_fields % where) + order_by
 	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd}], get_col_idx = True)
+	if return_pks:
+		return [ r['pk_billable'] for r in rows ]
 	return [ cBillable(row = {'data': r, 'idx': idx, 'pk_field': 'pk_billable'}) for r in rows ]
 
 #------------------------------------------------------------
@@ -127,7 +129,7 @@ def create_billable(code=None, term=None, data_source=None, return_existing=Fals
 		'term': term.strip(),
 		'data_src': data_source
 	}
-	cmd = u"""
+	cmd = """
 		INSERT INTO ref.billable (code, term, fk_data_source)
 		SELECT
 			%(code)s,
@@ -150,7 +152,7 @@ def create_billable(code=None, term=None, data_source=None, return_existing=Fals
 	if not return_existing:
 		return None
 
-	cmd = u"""
+	cmd = """
 		SELECT * FROM ref.v_billables
 		WHERE
 			code = %(code)s
@@ -164,7 +166,7 @@ def create_billable(code=None, term=None, data_source=None, return_existing=Fals
 
 #------------------------------------------------------------
 def delete_billable(pk_billable=None):
-	cmd = u"""
+	cmd = """
 		DELETE FROM ref.billable
 		WHERE
 			pk = %(pk)s
@@ -185,7 +187,7 @@ class cBillItem(gmBusinessDBObject.cBusinessDBObject):
 
 	_cmd_fetch_payload = _SQL_get_bill_item_fields % u"pk_bill_item = %s"
 	_cmds_store_payload = [
-		u"""UPDATE bill.bill_item SET
+		"""UPDATE bill.bill_item SET
 				fk_provider = %(pk_provider)s,
 				fk_encounter = %(pk_encounter_to_bill)s,
 				date_to_bill = %(raw_date_to_bill)s,
@@ -216,7 +218,7 @@ class cBillItem(gmBusinessDBObject.cBusinessDBObject):
 	]
 	#--------------------------------------------------------
 	def format(self):
-		txt = u'%s (%s %s%s)         [#%s]\n' % (
+		txt = '%s (%s %s%s)         [#%s]\n' % (
 			gmTools.bool2subst(
 				self._payload[self._idx['pk_bill']] is None,
 				_('Open item'),
@@ -224,25 +226,25 @@ class cBillItem(gmBusinessDBObject.cBusinessDBObject):
 			),
 			self._payload[self._idx['catalog_short']],
 			self._payload[self._idx['catalog_version']],
-			gmTools.coalesce(self._payload[self._idx['catalog_language']], u'', ' - %s'),
+			gmTools.coalesce(self._payload[self._idx['catalog_language']], '', ' - %s'),
 			self._payload[self._idx['pk_bill_item']]
 		)
-		txt += u' %s: %s\n' % (
+		txt += ' %s: %s\n' % (
 			self._payload[self._idx['billable_code']],
 			self._payload[self._idx['billable_description']]
 		)
 		txt += gmTools.coalesce (
 			self._payload[self._idx['billable_comment']],
-			u'',
-			u'  (%s)\n',
+			'',
+			'  (%s)\n',
 		)
 		txt += gmTools.coalesce (
 			self._payload[self._idx['item_detail']],
-			u'',
+			'',
 			_(' Details: %s\n'),
 		)
 
-		txt += u'\n'
+		txt += '\n'
 		txt += _(' %s of units: %s\n') % (
 			gmTools.u_numero,
 			self._payload[self._idx['unit_count']]
@@ -261,7 +263,7 @@ class cBillItem(gmBusinessDBObject.cBusinessDBObject):
 			'vat': self._payload[self._idx['vat']]
 		}
 
-		txt += u'\n'
+		txt += '\n'
 		txt += _(' Charge date: %s') % gmDateTime.pydt_strftime (
 			self._payload[self._idx['date_to_bill']],
 			'%Y %b %d',
@@ -290,20 +292,22 @@ class cBillItem(gmBusinessDBObject.cBusinessDBObject):
 
 	is_in_use = property(_get_is_in_use, lambda x:x)
 #------------------------------------------------------------
-def get_bill_items(pk_patient=None, non_invoiced_only=False):
+def get_bill_items(pk_patient=None, non_invoiced_only=False, return_pks=False):
 	if non_invoiced_only:
 		cmd = _SQL_get_bill_item_fields % u"pk_patient = %(pat)s AND pk_bill IS NULL"
 	else:
 		cmd = _SQL_get_bill_item_fields % u"pk_patient = %(pat)s"
 	args = {'pat': pk_patient}
 	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
+	if return_pks:
+		return [ r['pk_bill_item'] for r in rows ]
 	return [ cBillItem(row = {'data': r, 'idx': idx, 'pk_field': 'pk_bill_item'}) for r in rows ]
 
 #------------------------------------------------------------
 def create_bill_item(pk_encounter=None, pk_billable=None, pk_staff=None):
 
 	billable = cBillable(aPK_obj = pk_billable)
-	cmd = u"""
+	cmd = """
 		INSERT INTO bill.bill_item (
 			fk_provider,
 			fk_encounter,
@@ -330,21 +334,21 @@ def create_bill_item(pk_encounter=None, pk_billable=None, pk_staff=None):
 
 #------------------------------------------------------------
 def delete_bill_item(link_obj=None, pk_bill_item=None):
-	cmd = u'DELETE FROM bill.bill_item WHERE pk = %(pk)s AND fk_bill IS NULL'
+	cmd = 'DELETE FROM bill.bill_item WHERE pk = %(pk)s AND fk_bill IS NULL'
 	args = {'pk': pk_bill_item}
 	gmPG2.run_rw_queries(link_obj = link_obj, queries = [{'cmd': cmd, 'args': args}])
 
 #============================================================
 # bills
 #------------------------------------------------------------
-_SQL_get_bill_fields = u"""SELECT * FROM bill.v_bills WHERE %s"""
+_SQL_get_bill_fields = """SELECT * FROM bill.v_bills WHERE %s"""
 
 class cBill(gmBusinessDBObject.cBusinessDBObject):
 	"""Represents a bill"""
 
-	_cmd_fetch_payload = _SQL_get_bill_fields % u"pk_bill = %s"
+	_cmd_fetch_payload = _SQL_get_bill_fields % "pk_bill = %s"
 	_cmds_store_payload = [
-		u"""UPDATE bill.bill SET
+		"""UPDATE bill.bill SET
 				invoice_id = gm.nullify_empty_string(%(invoice_id)s),
 				close_date = %(close_date)s,
 				apply_vat = %(apply_vat)s,
@@ -362,17 +366,17 @@ class cBill(gmBusinessDBObject.cBusinessDBObject):
 		"""
 	]
 	_updatable_fields = [
-		u'invoice_id',
-		u'pk_receiver_identity',
-		u'close_date',
-		u'apply_vat',
-		u'comment',
-		u'pk_receiver_address',
-		u'pk_doc'
+		'invoice_id',
+		'pk_receiver_identity',
+		'close_date',
+		'apply_vat',
+		'comment',
+		'pk_receiver_address',
+		'pk_doc'
 	]
 	#--------------------------------------------------------
 	def format(self, include_receiver=True, include_doc=True):
-		txt = u'%s                       [#%s]\n' % (
+		txt = '%s                       [#%s]\n' % (
 			gmTools.bool2subst (
 				(self._payload[self._idx['close_date']] is None),
 				_('Open bill'),
@@ -422,18 +426,18 @@ class cBill(gmBusinessDBObject.cBusinessDBObject):
 				gmTools.bool2subst (
 					self._payload[self._idx['pk_doc']] is None,
 					_('not available'),
-					u'#%s' % self._payload[self._idx['pk_doc']]
+					'#%s' % self._payload[self._idx['pk_doc']]
 				)
 			)
 		txt += _(' Patient: #%s\n') % self._payload[self._idx['pk_patient']]
 		if include_receiver:
 			txt += gmTools.coalesce (
 				self._payload[self._idx['pk_receiver_identity']],
-				u'',
+				'',
 				_(' Receiver: #%s\n')
 			)
 			if self._payload[self._idx['pk_receiver_address']] is not None:
-				txt += u'\n '.join(gmDemographicRecord.get_patient_address(pk_patient_address = self._payload[self._idx['pk_receiver_address']]).format())
+				txt += '\n '.join(gmDemographicRecord.get_patient_address(pk_patient_address = self._payload[self._idx['pk_receiver_address']]).format())
 
 		return txt
 	#--------------------------------------------------------
@@ -471,7 +475,7 @@ class cBill(gmBusinessDBObject.cBusinessDBObject):
 	def _get_default_address(self):
 		return gmDemographicRecord.get_patient_address_by_type (
 			pk_patient = self._payload[self._idx['pk_patient']],
-			adr_type = u'billing'
+			adr_type = 'billing'
 		)
 
 	default_address = property(_get_default_address, lambda x:x)
@@ -479,7 +483,7 @@ class cBill(gmBusinessDBObject.cBusinessDBObject):
 	def _get_home_address(self):
 		return gmDemographicRecord.get_patient_address_by_type (
 			pk_patient = self._payload[self._idx['pk_patient']],
-			adr_type = u'home'
+			adr_type = 'home'
 		)
 
 	home_address = property(_get_home_address, lambda x:x)
@@ -496,35 +500,37 @@ class cBill(gmBusinessDBObject.cBusinessDBObject):
 		return self.save_payload()
 
 #------------------------------------------------------------
-def get_bills(order_by=None, pk_patient=None):
+def get_bills(order_by=None, pk_patient=None, return_pks=False):
 
 	args = {'pat': pk_patient}
-	where_parts = [u'true']
+	where_parts = ['true']
 
 	if pk_patient is not None:
-		where_parts.append(u'pk_patient = %(pat)s')
+		where_parts.append('pk_patient = %(pat)s')
 
 	if order_by is None:
-		order_by = u''
+		order_by = ''
 	else:
-		order_by = u' ORDER BY %s' % order_by
+		order_by = ' ORDER BY %s' % order_by
 
-	cmd = (_SQL_get_bill_fields % u' AND '.join(where_parts)) + order_by
+	cmd = (_SQL_get_bill_fields % ' AND '.join(where_parts)) + order_by
 	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
+	if return_pks:
+		return [ r['pk_bill'] for r in rows ]
 	return [ cBill(row = {'data': r, 'idx': idx, 'pk_field': 'pk_bill'}) for r in rows ]
 
 #------------------------------------------------------------
 def get_bills4document(pk_document=None):
 	args = {'pk_doc': pk_document}
-	cmd = _SQL_get_bill_fields % u'pk_doc = %(pk_doc)s'
+	cmd = _SQL_get_bill_fields % 'pk_doc = %(pk_doc)s'
 	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
 	return [ cBill(row = {'data': r, 'idx': idx, 'pk_field': 'pk_bill'}) for r in rows ]
 
 #------------------------------------------------------------
 def create_bill(conn=None, invoice_id=None):
 
-	args = {u'inv_id': invoice_id}
-	cmd = u"""
+	args = {'inv_id': invoice_id}
+	cmd = """
 		INSERT INTO bill.bill (invoice_id)
 		VALUES (gm.nullify_empty_string(%(inv_id)s))
 		RETURNING pk
@@ -536,7 +542,7 @@ def create_bill(conn=None, invoice_id=None):
 #------------------------------------------------------------
 def delete_bill(link_obj=None, pk_bill=None):
 	args = {'pk': pk_bill}
-	cmd = u"DELETE FROM bill.bill WHERE pk = %(pk)s"
+	cmd = "DELETE FROM bill.bill WHERE pk = %(pk)s"
 	gmPG2.run_rw_queries(link_obj = link_obj, queries = [{'cmd': cmd, 'args': args}])
 	return True
 
@@ -587,7 +593,6 @@ def generate_invoice_id(template=None, pk_patient=None, person=None, date_format
 		data['lastname'] = person['lastnames'].replace(' ', gmTools.u_space_as_open_box).strip()
 		data['dob'] = person.get_formatted_dob (
 			format = date_format,
-			encoding = 'utf8',
 			none_string = u'?',
 			honor_estimation = False
 		).strip()
@@ -624,35 +629,199 @@ def generate_invoice_id(template=None, pk_patient=None, person=None, date_format
 	return candidate_invoice_id.replace(u'#counter#', '%s' % counter)
 
 #------------------------------------------------------------
-def lock_invoice_id(invoice_id):
-	_log.debug('locking invoice ID: %s', invoice_id)
-	crc32 = zlib.crc32(invoice_id)
-	adler32 = zlib.adler32(invoice_id)
-	_log.debug('crc32: %s', crc32)
-	_log.debug('adler32: %s', adler32)
-	cmd = u"""SELECT pg_try_advisory_lock(%s, %s)""" % (crc32, adler32)
-	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd}])
+#------------------------------------------------------------
+# Remaining problems with invoice ID locking:
+#
+# If you run a 1.8.0rc1 client the lock can overflow PG's pg_try_advisory_lock().
+#
+# If you run both 1.8.0rc1 and 1.7 (<1.7.9) and happen to lock at the same
+# time the lock may succeed when it should not, because crc32/adler32 return
+# different representationel ranges in py2 and py3.
+#
+# If you run 1.7 (<1.7.9) on both "Python < 2.6" and "Python 2.6 or beyond"
+# and happen to lock at the same time the lock may succeed when it should
+# not, because crc32/adler32 return results with signedness depending on
+# platform.
+#------------------------------------------------------------
+#
+# remove in 1.9 / DB v23:
+def __lock_invoice_id_1_7_legacy(invoice_id):
+	"""Get 1.7 legacy (<1.7.9) lock.
+
+	The fix is to *down*shift py3 checksums into the py2 result range.
+	How to do that was suggested by MRAB on the Python mailing list.
+
+		https://www.mail-archive.com/python-list@python.org/msg447989.html
+
+	Problems:
+	- on py2 < 2.6 (client 1.7) signedness inconsistent across platforms
+	- on py3 (client 1.8), range shifted by & 0xffffffff
+
+	Because both 1.7 (<1.7.9) and 1.8 can run against the same
+	database v22 we need to retain this legacy lock until DB v23.
+	"""
+	_log.debug('legacy locking invoice ID: %s', invoice_id)
+	py3_crc32 = zlib.crc32(bytes(invoice_id, 'utf8'))
+	py3_adler32 = zlib.adler32(bytes(invoice_id, 'utf8'))
+	signed_crc32 = py3_crc32 - (py3_crc32 & 0x80000000) * 2
+	signed_adler32 = py3_adler32 - (py3_adler32 & 0x80000000) * 2
+	_log.debug('crc32: %s (py3, unsigned) -> %s (py2.6+, signed)', py3_crc32, signed_crc32)
+	_log.debug('adler32: %s (py3, unsigned) -> %s (py2.6+, signed)', py3_adler32, signed_adler32)
+	cmd = u"""SELECT pg_try_advisory_lock(%s, %s)""" % (signed_crc32, signed_adler32)
+	try:
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd}])
+	except gmPG2.dbapi.ProgrammingError:
+		# should not happen
+		_log.exception('cannot lock invoice ID: [%s] (%s/%s)', invoice_id, signed_crc32, signed_adler32)
+		return False
+
 	if rows[0][0]:
 		return True
 
-	_log.warning('cannot lock invoice ID: [%s] (%s/%s)', invoice_id, crc32, adler32)
+	_log.error('cannot lock invoice ID: [%s] (%s/%s)', invoice_id, signed_crc32, signed_adler32)
+	return False
+
+#------------------------------------------------------------
+def lock_invoice_id(invoice_id):
+	"""Lock an invoice ID.
+
+	The lock taken is an exclusive advisory lock in PostgreSQL.
+
+	Because the data is short _and_ crc32/adler32 are fairly
+	weak we assume that collisions can be created "easily".
+	Therefore we apply both algorithms concurrently.
+	"""
+	_log.debug('locking invoice ID: %s', invoice_id)
+	# remove in 1.9 / DB v23:
+	if not __lock_invoice_id_1_7_legacy(invoice_id):
+		return False
+
+	# get py2/py3 compatible lock:
+	# - upshift py2 result by & 0xffffffff for signedness consistency
+	# - still use both crc32 and adler32 but chain the result of
+	#   the former into the latter so we can take advantage of
+	#   pg_try_advisory_lock(BIGINT)
+	unsigned_crc32 = zlib.crc32(bytes(invoice_id, 'utf8')) & 0xffffffff
+	_log.debug('unsigned crc32: %s', unsigned_crc32)
+	data4adler32 = u'%s---[%s]' % (invoice_id, unsigned_crc32)
+	_log.debug('data for adler32: %s', data4adler32)
+	unsigned_adler32 = zlib.adler32(bytes(data4adler32, 'utf8'), unsigned_crc32) & 0xffffffff
+	_log.debug('unsigned (crc32-chained) adler32: %s', unsigned_adler32)
+	cmd = u"SELECT pg_try_advisory_lock(%s)" % (unsigned_adler32)
+	try:
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd}])
+	except gmPG2.dbapi.ProgrammingError:
+		_log.exception('cannot lock invoice ID: [%s] (%s)', invoice_id, unsigned_adler32)
+		return False
+
+	if rows[0][0]:
+		return True
+
+	_log.error('cannot lock invoice ID: [%s] (%s)', invoice_id, unsigned_adler32)
+	return False
+
+#------------------------------------------------------------
+def __unlock_invoice_id_1_7_legacy(invoice_id):
+	_log.debug('legacy unlocking invoice ID: %s', invoice_id)
+	py3_crc32 = zlib.crc32(bytes(invoice_id, 'utf8'))
+	py3_adler32 = zlib.adler32(bytes(invoice_id, 'utf8'))
+	signed_crc32 = py3_crc32 - (py3_crc32 & 0x80000000) * 2
+	signed_adler32 = py3_adler32 - (py3_adler32 & 0x80000000) * 2
+	_log.debug('crc32: %s (py3, unsigned) -> %s (py2.6+, signed)', py3_crc32, signed_crc32)
+	_log.debug('adler32: %s (py3, unsigned) -> %s (py2.6+, signed)', py3_adler32, signed_adler32)
+	cmd = u"""SELECT pg_advisory_unlock(%s, %s)""" % (signed_crc32, signed_adler32)
+	try:
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd}])
+	except gmPG2.dbapi.ProgrammingError:
+		_log.exception('cannot unlock invoice ID: [%s] (%s/%s)', invoice_id, signed_crc32, signed_adler32)
+		return False
+
+	if rows[0][0]:
+		return True
+
+	_log.error('cannot unlock invoice ID: [%s] (%s/%s)', invoice_id, signed_crc32, signed_adler32)
 	return False
 
 #------------------------------------------------------------
 def unlock_invoice_id(invoice_id):
 	_log.debug('unlocking invoice ID: %s', invoice_id)
-	crc32 = zlib.crc32(invoice_id)
-	adler32 = zlib.adler32(invoice_id)
-	_log.debug('crc32: %s', crc32)
-	_log.debug('adler32: %s', adler32)
-	cmd = u"""SELECT pg_advisory_unlock(%s, %s)""" % (crc32, adler32)
-	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd}])
+	# remove in 1.9 / DB v23:
+	if not __unlock_invoice_id_1_7_legacy(invoice_id):
+		return False
+
+	# unlock
+	unsigned_crc32 = zlib.crc32(bytes(invoice_id, 'utf8')) & 0xffffffff
+	_log.debug('unsigned crc32: %s', unsigned_crc32)
+	data4adler32 = u'%s---[%s]' % (invoice_id, unsigned_crc32)
+	_log.debug('data for adler32: %s', data4adler32)
+	unsigned_adler32 = zlib.adler32(bytes(data4adler32, 'utf8'), unsigned_crc32) & 0xffffffff
+	_log.debug('unsigned (crc32-chained) adler32: %s', unsigned_adler32)
+	cmd = u"SELECT pg_advisory_unlock(%s)" % (unsigned_adler32)
+	try:
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd}])
+	except gmPG2.dbapi.ProgrammingError:
+		_log.exception('cannot unlock invoice ID: [%s] (%s)', invoice_id, unsigned_adler32)
+		return False
+
 	if rows[0][0]:
 		return True
 
-	_log.warning('cannot unlock invoice ID: [%s] (%s/%s)', invoice_id, crc32, adler32)
+	_log.error('cannot unlock invoice ID: [%s] (%s)', invoice_id, unsigned_adler32)
 	return False
 
+#------------------------------------------------------------
+def get_scan2pay_data(branch, bill, provider=None, comment=None):
+	"""Create scan2pay data for generating a QR code.
+
+	https://www.scan2pay.info
+	--------------------------------
+	BCD														# (3) fixed, barcode tag
+	002														# (3) fixed, version
+	1														# (1) charset, 1 = utf8
+	SCT														# (3) fixed
+	$<praxis_id::BIC//Bank//%(value)s::11>$					# (11) <BIC>
+	$2<range_of::$<current_provider_name::%(lastnames)s::>$,$<praxis::%(praxis)s::>$::70>2$			# (70) <Name of beneficiary> "Empfänger" - Praxis
+	$<praxis_id::IBAN//Bank//%(value)s::34>$				# (34) <IBAN>
+	EUR$<bill::%(total_amount_with_vat)s::12>$				# (12) <Amount in EURO> "EUR12.5"
+															# (4) <purpose of transfer> - leer
+															# (35) <remittance info - struct> - only this XOR the next field - GNUmed: leer
+	$2<range_of::InvID=$<bill::%(invoice_id)s::>$/Date=$<today::%d.%B %Y::>$::140$>2$	# (140) <remittance info - text> "Client:Marie Louise La Lune" - "Rg Nr, date"
+	<beneficiary-to-payor info>								# (70)	"pay soon :-)" - optional - GNUmed nur wenn bytes verfügbar
+	--------------------------------
+	total: 331 bytes (not chars ! - cave UTF8)
+	EOL: LF or CRLF
+	last *used* element not followed by anything, IOW can omit pending non-used elements
+	"""
+	assert (branch is not None), '<branch> must not be <None>'
+	assert (bill is not None), '<bill> must not be <None>'
+
+	data = {}
+	IBANs = branch.get_external_ids(id_type = 'IBAN', issuer = 'Bank')
+	if len(IBANs) == 0:
+		_log.debug('no IBAN found, cannot create scan2pay data')
+		return None
+	data['IBAN'] = IBANs[0]['value'][:34]
+	data['beneficiary'] = gmTools.coalesce (
+		value2test = provider,
+		return_instead = branch['praxis'][:70],
+		template4value = '%%(lastnames)s, %s' % branch['praxis']
+	)[:70]
+	BICs = branch.get_external_ids(id_type = 'BIC', issuer = 'Bank')
+	if len(BICs) == 0:
+		data['BIC'] = ''
+	else:
+		data['BIC'] = BICs[0]['value'][:11]
+	data['amount'] = bill['total_amount_with_vat'][:9]
+	data['ref'] = (_('Inv: %s, %s') % (
+		bill['invoice_id'],
+		gmDateTime.pydt_strftime(gmDateTime.pydt_now_here(), '%d.%B %Y')
+	))[:140]
+	data['cmt'] = gmTools.coalesce(comment, '', '\n%s')[:70]
+
+	data_str = 'BCD\n002\n1\nSCT\n%(BIC)s\n%(beneficiary)s\n%(IBAN)s\nEUR%(amount)s\n\n\n%(ref)s%(cmt)s' % data
+	data_str_bytes = bytes(data_str, 'utf8')[:331]
+	return str(data_str_bytes, 'utf8')
+
 #============================================================
 # main
 #------------------------------------------------------------
@@ -666,26 +835,39 @@ if __name__ == "__main__":
 
 #	from Gnumed.pycommon import gmLog2
 #	from Gnumed.pycommon import gmI18N
+	from Gnumed.pycommon import gmDateTime
 #	from Gnumed.business import gmPerson
+	from Gnumed.business import gmPraxis
 
 #	gmI18N.activate_locale()
-##	gmDateTime.init()
+	gmDateTime.init()
 
 	def test_default_address():
 		bills = get_bills(pk_patient = 12)
 		first_bill = bills[0]
-		print first_bill.default_address
+		print(first_bill.default_address)
 
 	#--------------------------------------------------
 	def test_me():
-		print "--------------"
+		print("--------------")
 		me = cBillable(aPK_obj=1)
 		fields = me.get_fields()
 		for field in fields:
-			print field, ':', me[field]
-		print "updatable:", me.get_updatable_fields()
+			print(field, ':', me[field])
+		print("updatable:", me.get_updatable_fields())
 		#me['vat']=4; me.store_payload()
 
+	#--------------------------------------------------
+	def test_get_scan2pay_data():
+		prax = gmPraxis.get_praxis_branches()[0]
+		bills = get_bills(pk_patient = 12)
+		print(get_scan2pay_data (
+			prax,
+			bills[0],
+			provider=None,
+			comment = 'GNUmed test harness' + ('x' * 400)
+		))
+
 	#--------------------------------------------------
 	def test_generate_invoice_id():
 		from Gnumed.pycommon import gmI18N
@@ -693,11 +875,11 @@ if __name__ == "__main__":
 		gmI18N.install_domain()
 		import gmPerson
 		for idx in range(1,15):
-			print ''
-			print 'classic:', generate_invoice_id(pk_patient = idx)
+			print ('')
+			print ('classic:', generate_invoice_id(pk_patient = idx))
 			pat = gmPerson.cPerson(idx)
 			template = u'%(firstname).4s%(lastname).4s%(date)s'
-			print 'new: template = "%s" => %s' % (
+			print ('new: template = "%s" => %s' % (
 				template,
 				generate_invoice_id (
 					template = template,
@@ -706,23 +888,24 @@ if __name__ == "__main__":
 					date_format='%d%m%Y',
 					time_format='%H%M%S'
 				)
-			)
+			))
 			template = u'%(firstname).4s%(lastname).4s%(date)s-#counter#'
-			print 'new: template = "%s" => %s' % (
-				template,
-				generate_invoice_id (
-					template = template,
-					pk_patient = None,
-					person = pat,
-					date_format='%d%m%Y',
-					time_format='%H%M%S'
-				)
+			new_id = generate_invoice_id (
+				template = template,
+				pk_patient = None,
+				person = pat,
+				date_format='%d%m%Y',
+				time_format='%H%M%S'
 			)
+			print('locked: %s' % lock_invoice_id(new_id))
+			print ('new: template = "%s" => %s' % (template, new_id))
+			print('unlocked: %s' % unlock_invoice_id(new_id))
 
 		#generate_invoice_id(template=None, pk_patient=None, person=None, date_format='%Y-%m-%d', time_format='%H%M%S')
 
 	#--------------------------------------------------
 
 	#test_me()
-	test_default_address()
+	#test_default_address()
+	#test_get_scan2pay_data()
 	test_generate_invoice_id()
