@@ -711,7 +711,7 @@ def format_pregnancy_weeks(age):
 	return '%s%s%s%s' % (
 		int(weeks),
 		_('interval_format_tag::weeks::w')[-1:],
-		interval.days,
+		days,
 		_('interval_format_tag::days::d')[-1:]
 	)
 
@@ -1700,11 +1700,12 @@ def __numbers_only(str2parse):
 	if not regex.match("^(\s|\t)*\d{1,4}(\s|\t)*$", str2parse, flags = regex.UNICODE):
 		return []
 
-	now = pydt_now_here()
 	val = int(regex.findall('\d{1,4}', str2parse, flags = regex.UNICODE)[0])
+	if val == 0:
+		return []
 
+	now = pydt_now_here()
 	matches = []
-
 	# today in that year
 	if (1850 < val) and (val < 2100):
 		target_date = cFuzzyTimestamp (
@@ -2385,7 +2386,7 @@ if __name__ == '__main__':
 	#-------------------------------------------------
 	def test_get_date_of_weekday_in_week_of_date():
 		dt = pydt_now_here()
-		print('weekday', base_dt.isoweekday(), '(2day):', dt)
+		print('weekday', dt.isoweekday(), '(2day):', dt)
 		for weekday in range(8):
 			dt = get_date_of_weekday_in_week_of_date(weekday)
 			print('weekday', weekday, '(same):', dt)
@@ -2400,6 +2401,15 @@ if __name__ == '__main__':
 		except ValueError as exc:
 			print(exc)
 
+	#-------------------------------------------------
+	def test__numbers_only():
+		for val in range(-1, 35):
+			matches = __numbers_only(str(val))
+			print(val, ':')
+			for m in matches:
+				print('  ', m)
+			input()
+
 	#-------------------------------------------------
 	# GNUmed libs
 	gmI18N.activate_locale()
@@ -2408,7 +2418,7 @@ if __name__ == '__main__':
 	init()
 
 	#test_date_time()
-	test_str2fuzzy_timestamp_matches()
+	#test_str2fuzzy_timestamp_matches()
 	#test_get_date_of_weekday_in_week_of_date()
 	#test_cFuzzyTimeStamp()
 	#test_get_pydt()
@@ -2419,5 +2429,6 @@ if __name__ == '__main__':
 	#test_pydt_strftime()
 	#test_calculate_apparent_age()
 	#test_is_leap_year()
+	test__numbers_only()
 
 #===========================================================================
