@@ -15,9 +15,11 @@
 # You should have received a copy of the GNU General Public License
 # along with Timeline.  If not, see <http://www.gnu.org/licenses/>.
 
+"""
+Defines plugin application components.
+"""
 
-from factory import PluginFactory
 
+from timelinelib.plugin.factory import PluginFactory
 
 factory = PluginFactory()
-factory.load_plugins()
