@@ -29,7 +29,7 @@ Part 1:  http://gnumed.blogspot.com/2010/04/gnumed-plugin-development-how-to-sha
 Part 2:  http://gnumed.blogspot.com/2010/07/gnumed-plugin-development-easy-testing.html
 
 For development information such as database schema, function and classes documentation
-and more see http://wiki.gnumed.de
+and more see https://www.gnumed.de/documentation/
 """
 
 """
@@ -115,7 +115,7 @@ if __name__ == '__main__':
 	# obtain patient
 	patient = gmPersonSearch.ask_for_patient()
 	if patient is None:
-		print "None patient. Exiting gracefully..."
+		print("No patient. Exiting gracefully...")
 		sys.exit(0)
 	gmPatSearchWidgets.set_active_patient(patient=patient)
 
@@ -131,6 +131,6 @@ if __name__ == '__main__':
 		try:
 			patient.cleanup()
 		except Exception:
-			print "error cleaning up patient"
+			print("error cleaning up patient")
 
 	_log.info("closing example plugin...")
